/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.emf;

import java.io.IOException;
import java.io.InputStream;

import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.FCSourceCodeGeneratorItf;
import org.objectweb.fractal.juliac.InitializerClassGenerator;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.conf.JuliacConfig;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.ProxyClassGenerator;

/**
 * This class generates the source code associated to Fractal components.
 * The membrane implementation and the initializer implementation are generated.
 * 
 * The content, the interceptors and the controllers are kept in separate
 * classes and the controllers are implemented with objects.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.1
 */
public class EMFParserSourceCodeGenerator
implements FCSourceCodeGeneratorItf<Void> {
    
    public void init( Juliac jc ) {
        this.jc = jc;
    }
    private Juliac jc;
    
    public boolean acceptADLDesc( String adl ) {
        String adlFileName = adl.replace('.','/')+".composite";
        JuliacConfig jconf = jc.getJuliacConfig();
        ClassLoader loader = jconf.getClassLoader();
        InputStream is = loader.getResourceAsStream(adlFileName);
        boolean exist = (is != null);
        return exist;
    }
    
    public boolean acceptCtrlDesc( Object controllerDesc ) {
        /*
         * No controller descriptor accepted (only ADL descriptors can be
         * handled by this optimization level source code generator.
         */
        return false;
    }

    /**
     * Generate the source code of a factory class for the specified ADL.
     * 
     * @param adl         the fully-qualified name of the ADL descriptor
     * @param targetname  the fully-qualified name of the factory class
     */
    public void generate( String adl, String targetname ) throws IOException {
        EMFParserSupportImpl emf = new EMFParserSupportImpl();
        emf.init(jc);
        emf.generate(adl,targetname);
    }

    public void close() {
        // Indeed nothing
    }

    public InitializerClassGenerator<Void> generate(
        Type type, Object controllerDesc, Object contentDesc ) {
        
        String msg = "Not relevant for this optimization level";
        throw new UnsupportedOperationException(msg);
    }

    public MembraneDesc<Void> generateMembraneImpl(
        ComponentType ct, String ctrlDesc, String contentClassName ) {
        
        String msg = "Not relevant for this optimization level";
        throw new UnsupportedOperationException(msg);
    }

    public ProxyClassGenerator getInterfaceClassGenerator(
        InterfaceType it, MembraneDesc<?> membraneDesc ) {
        
        String msg = "Not relevant for this optimization level";
        throw new UnsupportedOperationException(msg);
    }
}
